/*
 * asterisk -- An open source telephony toolkit.
 *
 * Copyright (C) 2019, Digium, Inc.
 *
 * Brad Latus <brad.latus@gmail.com>
 *
 * See http://www.asterisk.org for more information about
 * the Asterisk project. Please do not directly contact
 * any of the maintainers of this project for assistance;
 * the project provides a web site, mailing lists and IRC
 * channels for your use.
 *
 * This program is free software, distributed under the terms of
 * the GNU General Public License Version 2. See the LICENSE file
 * at the top of the source tree.
 */

/*** MODULEINFO
	<depend>pjproject</depend>
	<depend>res_pjsip</depend>
	<depend>res_pjsip_pubsub</depend>
	<support_level>core</support_level>
 ***/

#include "asterisk.h"

#include <pjsip.h>
#include <pjsip_simple.h>
#include <pjlib.h>

#include "asterisk/module.h"
#include "asterisk/res_pjsip.h"
#include "asterisk/res_pjsip_pubsub.h"
#include "asterisk/res_pjsip_presence_xml.h"
#include "asterisk/res_pjsip_body_generator_types.h"

/*!
 * \internal
 * \brief Adds non standard elements to the xml body
 *
 * This is Cisco definition for the PIDF format
 */
static void add_cisco(pj_pool_t *pool, pj_xml_node *node, const char *pidfstate)
{
	//static const char *XMLNS_PREFIX = "xmlns";
	//static const char *XMLNS = "urn:ietf:params:xml:ns:pidf";

	static const char *XMLNS_E_PREFIX = "xmlns:e";
	static const char *XMLNS_E = "urn:ietf:params:xml:ns:pidf:status:rpid";

	static const char *XMLNS_CE_PREFIX = "xmlns:ce";
	static const char *XMLNS_CE = "urn:cisco:params:xml:ns:pidf:rpid";

	pj_xml_node *person = ast_sip_presence_xml_create_node(pool, node, "dm:person");

	if (pidfstate[0] != '-') {
		pj_xml_node *activities = ast_sip_presence_xml_create_node(pool, person, "e:activities");
		size_t str_size = sizeof("e:") + strlen(pidfstate);
		char *act_str = ast_alloca(str_size);

		/* Safe */
		strcpy(act_str, "e:");
		strcat(act_str, pidfstate);

		ast_sip_presence_xml_create_node(pool, activities, act_str);
	}

	//ast_sip_presence_xml_create_attr(pool, node, XMLNS_PREFIX, XMLNS);
	ast_sip_presence_xml_create_attr(pool, node, XMLNS_E_PREFIX, XMLNS_E);
	ast_sip_presence_xml_create_attr(pool, node, XMLNS_CE_PREFIX, XMLNS_CE);
}

static int pidf_supplement_body(void *body, void *data)
{
	pjpidf_pres *pres = body;
	struct ast_sip_exten_state_data *state_data = data;
	char *statestring = NULL, *pidfstate = NULL, *pidfnote = NULL;
	enum ast_sip_pidf_state local_state;

	if (ast_strlen_zero(state_data->user_agent) ||
	    !strstr(state_data->user_agent, "cisco")) {
		/* not a digium phone */
	//	ast_verb(3,"Not Cisco: %s\n",state_data->user_agent);

		return 0;
	}
	//ast_verb(3,"Cisco Phone here\n");
	ast_sip_presence_exten_state_to_str(state_data->exten_state, &statestring,
			&pidfstate, &pidfnote, &local_state, 0);

	add_cisco(state_data->pool, pres, pidfstate);
	return 0;
}

static struct ast_sip_pubsub_body_supplement pidf_supplement = {
	.type = "application",
	.subtype = "pidf+xml",
	.supplement_body = pidf_supplement_body,
};

static int load_module(void)
{
	if (ast_sip_pubsub_register_body_supplement(&pidf_supplement)) {
		return AST_MODULE_LOAD_DECLINE;
	}
	return AST_MODULE_LOAD_SUCCESS;
}

static int unload_module(void)
{
	ast_sip_pubsub_unregister_body_supplement(&pidf_supplement);
	return 0;
}

AST_MODULE_INFO(ASTERISK_GPL_KEY, AST_MODFLAG_LOAD_ORDER, "PJSIP PIDF Cisco supplement",
	.support_level = AST_MODULE_SUPPORT_CORE,
	.load = load_module,
	.unload = unload_module,
	.load_pri = AST_MODPRI_CHANNEL_DEPEND,
	.requires = "res_pjsip,res_pjsip_pubsub",
);
